@@ -137,15 +137,32 @@ int ne_version_pre_http11(ne_session *s)
 /* Stores the "hostname[:port]" segment */
 static void set_hostport(struct host_info *host, unsigned int defaultport)
 {
-    size_t len = strlen(host->hostname);
-    host->hostport = ne_malloc(len + 10);
-    strcpy(host->hostport, host->hostname);
-    if (host->port != defaultport)
-	ne_snprintf(host->hostport + len, 9, ":%u", host->port);
+    if (host->port == defaultport) {
+        host->hostport = ne_strdup(host->hostname);
+    }
+    else {
+        char buf[512];
+
+        ne_snprintf(buf, sizeof buf, "%s:%u", host->hostname, host->port);
+        host->hostport = ne_strdup(buf);
+    }
 }
 
-/* Stores the hostname/port in *info, setting up the "hostport"
- * segment correctly. */
+#define V6_ADDR_MINLEN strlen("[::1]") /* "[::]" never valid */
+#define V6_SCOPE_SEP "%25"
+#define V6_SCOPE_SEPLEN (strlen(V6_SCOPE_SEP))
+/* Minimum length of link-local address with scope. */
+#define V6_SCOPE_MINLEN (strlen("[fe80::%251]"))
+
+/* Stores the hostname/port in *HI, setting up the "hostport" segment
+ * correctly. RFC 6874 syntax is allowed here but the scope ID is
+ * stripped from the hostname which is used in the Host header.  RFC
+ * 9110's Host header uses uri-host, which references RFC 3986 and not
+ * RFC 6874, so it is pedantically correct; the scope ID also has no
+ * possible interpretation outside of the client host.
+ *
+ * TODO: This function also does not propagate parse failures or scope
+ * mapping failures, which is bad. */
 static void set_hostinfo(struct host_info *hi, enum proxy_type type, 
                          const char *hostname, unsigned int port)
 {
@@ -158,15 +175,48 @@ static void set_hostinfo(struct host_info *hi, enum proxy_type type,
 
     hlen = strlen(hi->hostname);
 
-    /* IP literal parsing */
+    /* IP literal parsing. */
     ia = ne_iaddr_parse(hi->hostname, ne_iaddr_ipv4);
-    if (!ia && hlen > 4
+    if (!ia && hlen >= V6_ADDR_MINLEN
         && hi->hostname[0] == '[' && hi->hostname[hlen-1] == ']') {
-        char *v6lit = ne_strdup(hi->hostname + 1);
+        const char *v6end, *v6start = hi->hostname + 1;
+        char *v6lit, *scope = NULL;
 
-        v6lit[hlen-2] = '\0';
+        /* Parse here, see if there is a Zone ID:
+         *  IPv6addrzb => v6start = IPv6address "%25" ZoneID */
 
+        if (hlen >= V6_SCOPE_MINLEN
+            && (scope = strstr(v6start, V6_SCOPE_SEP)) != NULL)
+            v6end = scope;
+        else
+            v6end = hi->hostname + hlen - 1; /* trailing ']' */
+
+        /* Extract the IPv6-literal part. */
+        v6lit = ne_strndup(v6start, v6end - v6start);
         ia = ne_iaddr_parse(v6lit, ne_iaddr_ipv6);
+        if (ia && scope) {
+            /* => scope = "%25" scope  "]" */
+            char *v6scope = ne_strndup(scope + V6_SCOPE_SEPLEN,
+                                       strlen(scope) - (V6_SCOPE_SEPLEN + 1));
+
+            if (ne_iaddr_set_scope(ia, v6scope) == 0) {
+                /* Strip scope from hostname since it's used in Host:
+                 * headers and will be rejected. This is safe since
+                 * strlen(scope) is assured by strstr() above. */
+                *scope++ = ']';
+                *scope = '\0';
+                NE_DEBUG(NE_DBG_HTTP, "sess: Using IPv6 scope '%s', "
+                         "hostname rewritten to %s.\n", v6scope,
+                         hi->hostname);
+            }
+            else {
+                NE_DEBUG(NE_DBG_HTTP, "sess: Failed to set IPv6 scope '%s' "
+                         "for address %s.\n", v6scope, v6lit);
+            }
+
+            ne_free(v6scope);
+        }
+
         ne_free(v6lit);
     }
 
@@ -185,7 +235,7 @@ ne_session *ne_session_create(const char *scheme,
     NE_DEBUG(NE_DBG_HTTP, "HTTP session to %s://%s:%d begins.\n",
 	     scheme, hostname, port);
 
-    strcpy(sess->error, "Unknown error.");
+    ne_strnzcpy(sess->error, _("Unknown error."), sizeof sess->error);
 
     /* use SSL if scheme is https */
     sess->use_ssl = !strcmp(scheme, "https");
@@ -197,7 +247,6 @@ ne_session *ne_session_create(const char *scheme,
 #ifdef NE_HAVE_SSL
     if (sess->use_ssl) {
         sess->ssl_context = ne_ssl_context_create(0);
-        sess->flags[NE_SESSFLAG_SSLv2] = 1;
         
         if (!sess->server.literal) {
             sess->flags[NE_SESSFLAG_TLS_SNI] = 1;
@@ -212,6 +261,7 @@ ne_session *ne_session_create(const char *scheme,
 
     /* Set flags which default to on: */
     sess->flags[NE_SESSFLAG_PERSIST] = 1;
+    sess->flags[NE_SESSFLAG_STRICT] = 1;
 
 #ifdef NE_ENABLE_AUTO_LIBPROXY
     ne_session_system_proxy(sess, 0);
@@ -382,12 +432,6 @@ void ne_set_session_flag(ne_session *sess, ne_session_flag flag, int value)
 {
     if (flag < NE_SESSFLAG_LAST) {
         sess->flags[flag] = value;
-#ifdef NE_HAVE_SSL
-        if (flag == NE_SESSFLAG_SSLv2 && sess->ssl_context) {
-            ne_ssl_context_set_flag(sess->ssl_context, NE_SSL_CTX_SSLv2, value);
-            sess->flags[flag] = ne_ssl_context_get_flag(sess->ssl_context, NE_SSL_CTX_SSLv2);
-        }
-#endif
     }
 }
 
@@ -444,13 +488,7 @@ void ne_set_connect_timeout(ne_session *sess, int timeout)
 void ne_set_useragent(ne_session *sess, const char *token)
 {
     if (sess->user_agent) ne_free(sess->user_agent);
-    sess->user_agent = ne_malloc(strlen(UAHDR) + strlen(AGENT) + 
-                                 strlen(token) + 1);
-#ifdef HAVE_STPCPY
-    strcpy(stpcpy(stpcpy(sess->user_agent, UAHDR), token), AGENT);
-#else
-    strcat(strcat(strcpy(sess->user_agent, UAHDR), token), AGENT);
-#endif
+    sess->user_agent = ne_concat(UAHDR, token, AGENT, NULL);
 }
 
 const char *ne_get_server_hostport(ne_session *sess)
@@ -537,6 +575,35 @@ void ne_ssl_trust_cert(ne_session *sess, const ne_ssl_certificate *cert)
 #endif
 }
 
+int ne_ssl_set_protovers(ne_session *sess, enum ne_ssl_protocol min,
+                         enum ne_ssl_protocol max)
+{
+#ifdef NE_HAVE_SSL
+    if (sess->ssl_context) {
+        if (ne_ssl_context_set_versions(sess->ssl_context, min, max) != 0) {
+            ne_set_error(sess, _("Could not set minimum/maximum SSL/TLS versions"));
+            return NE_ERROR;
+        }
+
+        return NE_OK;
+    }
+#endif
+    ne_set_error(sess, _("SSL/TLS not enabled for the session"));
+    return NE_ERROR;
+}
+
+static const char *const ssl_proto_names[] = { "unknown", "SSLv3",
+                                               "TLSv1.0", "TLSv1.1",
+                                               "TLSv1.2", "TLSv1.3" };
+
+const char *ne_ssl_proto_name(enum ne_ssl_protocol proto)
+{
+    if (proto < (sizeof(ssl_proto_names)/sizeof(ssl_proto_names[0])))
+        return ssl_proto_names[proto];
+    else
+        return ssl_proto_names[0];
+}
+
 void ne_ssl_cert_validity(const ne_ssl_certificate *cert, char *from, char *until)
 {
 #ifdef NE_HAVE_SSL
