@@ -1,6 +1,6 @@
 /* 
    Stupidly simple test framework
-   Copyright (C) 2001-2002, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2001-2004, Joe Orton <joe@manyfish.co.uk>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -43,13 +43,26 @@ typedef int (*test_func)(void);
 typedef struct {
     test_func fn; /* the function to test. */
     const char *name; /* the name of the test. */
+    int flags;
 } ne_test;
 
+/* possible values for flags: */
+#define T_CHECK_LEAKS (1) /* check for memory leaks */
+#define T_EXPECT_FAIL (2) /* expect failure */
+#define T_EXPECT_LEAKS (4) /* expect memory leak failures */
+
 /* array of tests to run: must be defined by each test suite. */
 extern ne_test tests[];
 
-/* define a test function which has the same name as the function. */
-#define T(fn) { fn, #fn }
+/* define a test function which has the same name as the function,
+ * and does check for memory leaks. */
+#define T(fn) { fn, #fn, T_CHECK_LEAKS }
+/* define a test function which is expected to return FAIL. */
+#define T_XFAIL(fn) { fn, #fn, T_EXPECT_FAIL | T_CHECK_LEAKS }
+/* define a test function which isn't checked for memory leaks. */
+#define T_LEAKY(fn) { fn, #fn, 0 }
+/* define a test function which is expected to fail memory leak checks */
+#define T_XLEAKY(fn) { fn, #fn, T_EXPECT_LEAKS }
 
 /* current test number */
 extern int test_num;
@@ -101,11 +114,15 @@ void t_warning(const char *str, ...)
  * actual string for some field 'name'.  Succeeds if strcmp(exp,act)
  * == 0 or both are NULL. */
 #define ONCMP(exp, act, ctx, name) do { \
-ONV(exp && !act, ("%s: " name " was NULL, expected non-NULL", ctx)); \
-ONV(!exp && act, ("%s: " name " was non-NULL, expected NULL", ctx)); \
-ONV(exp && strcmp(exp, act), ("%s: " name " was %s not %s", ctx, exp, act)); \
+        ONV(exp && !act, ("%s: " name " was NULL, expected '%s'", ctx, exp)); \
+        ONV(!exp && act, ("%s: " name " was '%s', expected NULL", ctx, act)); \
+        ONV(exp && strcmp(exp, act), ("%s: " name " was '%s' not '%s'", ctx, act, exp)); \
 } while (0)
 
+/* Variant of ONCMP(), 'exp' and 'act' are known to be non-NULL. */
+#define ONCMPN(exp, act, ctx, name) \
+    ONV(strcmp(exp, act), ("%s: " name " was '%s' not '%s'", ctx, act, exp))
+
 /* return immediately with result of test 'x' if it fails. */
 #define CALL(x) do { int t_ret = (x); if (t_ret != OK) return t_ret; } while (0)
 
