@@ -321,6 +321,10 @@ static int parse(void)
         { "http://[::1]:8080/bar", "http", "[::1]", 8080, "/bar", NULL, NULL, NULL },
         { "ftp://[feed::cafe]:555", "ftp", "[feed::cafe]", 555, "/", NULL, NULL, NULL },
 
+        /* Test RFC 6874 syntax, an extension of RFC 3987. */
+        { "http://[fe80::cafe%25eth0]:555", "http", "[fe80::cafe%25eth0]", 555, "/", NULL, NULL, NULL },
+        { "http://[fe80::cafe%251]:555", "http", "[fe80::cafe%251]", 555, "/", NULL, NULL, NULL },
+
         { "DAV:", "DAV", NULL, 0, "", NULL, NULL, NULL },
         
         /* Some odd cases: heir-part and relative-ref will both match
@@ -380,6 +384,8 @@ static int failparse(void)
         "http://fish/[foo]/bar",
         "http://foo:80bar",
         "http://foo:80:80/bar",
+        "http://foo:8000000000000000000000000000000000000000000000000/bar",
+        "http://foo:65536/bar",
         NULL
     };
     int n;
